#include "Binary2C++.h" // see binary2c++.h for more info 

HINSTANCE g_hInstance;
HICON g_icoExe;

// entry point 
int APIENTRY  WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	// save instance handle for later use.
	g_hInstance=hInstance;

	// load icon for main dialog and about dialog
	g_icoExe=LoadIcon(hInstance, MAKEINTRESOURCE(ICO_EXE) );

	// used for Windows XP themes (Styles) support.
	InitCommonControls();

	// display main dialog window.
	DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), 0, (DLGPROC)DlgProc);
	
	// clean up stuff.
	DestroyIcon(g_icoExe);
	return 0;
}

// processes messages of main dialog.
LRESULT CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{

	case WM_INITDIALOG://sent before displaying the dialog.
		{	
			// set dialog icon. 
			SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_icoExe);

			// Set text limit to MAX_PATH
			SendMessage(GetDlgItem(hwndDlg,IDC_TXT_VAR),EM_LIMITTEXT,MAX_PATH,0);
			SendMessage(GetDlgItem(hwndDlg,IDC_TXT_BIN),EM_LIMITTEXT,MAX_PATH,0);
			SendMessage(GetDlgItem(hwndDlg,IDC_TXT_TARGET),EM_LIMITTEXT,MAX_PATH,0);

			return true;
		}
	case WM_COMMAND:
		{
			switch( wParam ) 
			{
			case IDC_CMD_BIN:// binary file path selection button clicked.
				GetFileName(hwndDlg,GetDlgItem(hwndDlg,IDC_TXT_BIN),TRUE);
				break;
			case IDC_CMD_TARGET:// target file path selection button clicked.
				GetFileName(hwndDlg,GetDlgItem(hwndDlg,IDC_TXT_TARGET),FALSE);
				break;
			case IDC_CMD_EXIT:// exit button clicked.
				EndDialog(hwndDlg,0);
				break;
			case IDC_CMD_ABOUT:// about button clicked.
				DialogBox(g_hInstance,MAKEINTRESOURCE(IDD_ABOUT),hwndDlg,(DLGPROC)AboutDlgProc);
				break;
			case IDC_CMD_CONVERT:// convert button clicked.
				Convert(hwndDlg);
				break;
			}
			break;
		}

	case WM_CLOSE:// close button clicked on title bar.
		{
			EndDialog(hwndDlg,0);// close the dialog (closing dialog cause exit).
			break;
		}


	}
	return FALSE;
}

// handle about dialog messages. 
LRESULT CALLBACK AboutDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{
	case WM_COMMAND:
		if ( wParam==IDC_CMD_OK)// ok button clicked on about dialog box
			EndDialog(hwndDlg,0);
		break;
	case WM_INITDIALOG:
		{	
			// set about dialog icon. 
			SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_icoExe);
			return true;
		}
	case WM_CLOSE:
		{
		EndDialog(hwndDlg,0);
			break;
		}	
	}
	return FALSE;
}

// show up open and save dialog.
void GetFileName(HWND hwnd,HWND hTextBox,BOOL bOpen)
{
	OPENFILENAME ofn;
	char szFile[MAX_PATH];



	ZeroMemory(&ofn, sizeof(ofn));
	szFile[0]=0;

	ofn.lpstrFile = szFile;
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;


	if (bOpen==TRUE) // select any file for binary.
	{
		szFile[0]=0;
		ofn.lpstrFile = szFile;
		ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST |OFN_HIDEREADONLY;
		ofn.lpstrTitle="Open...";
		ofn.lpstrFilter ="All Files\0*.*\0\0";

		if (GetOpenFileName(&ofn)==TRUE)
			SetWindowText(hTextBox,szFile);

	}
	else// get name of C++ header file.
	{
		GetDlgItemText(hwnd,IDC_TXT_VAR,szFile,MAX_PATH);
		ofn.lpstrFile = szFile;
		ofn.Flags = OFN_NODEREFERENCELINKS | OFN_OVERWRITEPROMPT;
		ofn.lpstrTitle="Save...";
		ofn.lpstrFilter ="C/C++ Header (*.h)\0*.h\0\0";
		ofn.lpstrDefExt="h";

		if (GetSaveFileName(&ofn)==TRUE)
			SetWindowText(hTextBox,szFile);
	}

}

void Convert(HWND hWnd)
{
	//**************************************************************
	// check if no text in binary file path(textbox).
	if (GetWindowTextLength(GetDlgItem(hWnd,IDC_TXT_BIN))==0)
	{
		MessageBox(hWnd,"Please select binary file.","Missing File Path",MB_OK |MB_ICONWARNING);
		return;
	}
	// check if no text in target file path(textbox).
	if (GetWindowTextLength(GetDlgItem(hWnd,IDC_TXT_TARGET))==0)
	{
		MessageBox(hWnd,"Please select target file.","Missing File Path",MB_OK |MB_ICONWARNING);
		return;
	}
	// check if variable name text is empty(textbox).
	if (GetWindowTextLength(GetDlgItem(hWnd,IDC_TXT_VAR))==0)
	{
		MessageBox(hWnd,"Please enter variable name.","Information",MB_OK |MB_ICONWARNING);
		return;
	}
	//**************************************************************
	//_____________________________________________________
	HANDLE hfBin,hfTarget;
	char szBinPath[MAX_PATH],szTargetPath[MAX_PATH];

	// get target file path and try to create new file.
	GetDlgItemText(hWnd,IDC_TXT_TARGET,szTargetPath,MAX_PATH);

	hfTarget = CreateFile(szTargetPath,
		GENERIC_WRITE,FILE_SHARE_READ,NULL,
		CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,
		NULL);

	if (hfTarget==INVALID_HANDLE_VALUE)
	{
		MessageBox(hWnd,"Error while creating new file." ,"Error",MB_OK |MB_ICONSTOP);
		return;
	}

	// get binary file path and Try to open binary file.
	GetDlgItemText(hWnd,IDC_TXT_BIN,szBinPath,MAX_PATH);

	hfBin = CreateFile(szBinPath,
		GENERIC_READ,FILE_SHARE_READ,NULL,
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,
		NULL);

	if (hfBin==INVALID_HANDLE_VALUE)
	{
		MessageBox(hWnd,"Error while opening binary file." ,"Error",MB_OK |MB_ICONSTOP);
		CloseHandle(hfTarget);
		return;
	}

	// check if binary file is empty.
	if (GetFileSize(hfBin,NULL)==0 )
	{
		CloseHandle(hfTarget);
		CloseHandle(hfBin);
		MessageBox(hWnd,"Binary file is empty.","Error",MB_OK |MB_ICONSTOP);
		return;
	}
	//_____________________________________________________
	// files are opened.
	char szVariableName[MAX_PATH];
	char szStr[2048],szTmp[MAX_PATH + 100];
	unsigned char ccBin[16];
	DWORD dwWritten,dwRead;
	BOOL bSuccess;


	// get variable name from text box.
	GetDlgItemText(hWnd,IDC_TXT_VAR,szVariableName,MAX_PATH);

	// fill target file header information in buffer string .
	szStr[0]=0;
	lstrcat(szStr,"// ");
	lstrcat(szStr,PathFindFileName(szTargetPath));
	lstrcat(szStr," generated with Binary2C++.");
	lstrcat(szStr,newLine);
	lstrcat(szStr,"// Binary2C++ developed by Muhammad Arshad Latti.");
	lstrcat(szStr,newLine);
	lstrcat(szStr,newLine);
	// make variable name and size in buffer string. 
	wsprintf(szTmp,"unsigned char %s[%lu]={",szVariableName,GetFileSize(hfBin,NULL));
	lstrcat(szStr,szTmp);

	// write header information to target file (from buffer string).
	WriteFile(hfTarget,szStr,lstrlen(szStr),&dwWritten,NULL);

	// read first byte of binary file.
	bSuccess = ReadFile(hfBin, ccBin, 1,&dwRead, 0);

	// convert first byte to hex value and write to target file.
	wsprintf(szStr,"0x%02X",ccBin[0]);
	WriteFile(hfTarget,szStr,lstrlen(szStr),&dwWritten,NULL);

	//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	do	// do a loop until end of binary file.
	{
		// try to read 16 bytes from binary file.
		bSuccess = ReadFile(hfBin, ccBin,sizeof(ccBin),&dwRead, 0);

		if (dwRead > 0)// if we get some bytes from binary file.
		{
			// convert 17 bytes to hex values(16 bytes used)
			// first make a new line (using "\xD\xA\")then replace 
			//first value startup space with one tab (using "x9").
			wsprintf(szStr,// buffer string
				"%s0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X",
				",\xD\xA\x9",ccBin[0],ccBin[1],ccBin[2],ccBin[3],ccBin[4],ccBin[5],ccBin[6],ccBin[7],ccBin[8],ccBin[9],ccBin[10],ccBin[11],ccBin[12],ccBin[13],ccBin[14],ccBin[15],ccBin[16]);

			// multiply bytes(number of read from binary file) with each hex value
			// size(6) and add 2 for new line characters ("\xD\xA\")
			// then terminate buffer string with zero(null). 
			szStr[(dwRead * 6)+2]=0;

			// write buffer string to target file.
			WriteFile(hfTarget,szStr,lstrlen(szStr),&dwWritten,NULL);
		}
	}
	while (dwRead>0 && bSuccess);
	//'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

	// add end data to buffer string and write it to target file.
	wsprintf(szStr,"%s};",newLine);
	WriteFile(hfTarget,szStr,lstrlen(szStr),&dwWritten,NULL);

	//Close both files.
	CloseHandle(hfTarget);
	CloseHandle(hfBin);

	// show up success message.
	MessageBox(hWnd,"Target file is written.","Success",MB_OK |MB_ICONINFORMATION);
}